from django.forms import ModelForm


class DynamicModelForm(ModelForm):

    def clean(self, *args, **kwargs):
        cleaned_data = super().clean(*args, **kwargs)
        if self.files.get(self.prefix + '-value'):
            cleaned_data['value'] = self.files.get(self.prefix + '-value')
        return cleaned_data



from PIL import Image, UnidentifiedImageError
from django.db import models
from django.conf import settings
from django.forms import MultiWidget, Textarea
from django.utils.safestring import mark_safe
from django.contrib.admin.widgets import AdminFileWidget


class DynamicWidget(MultiWidget):

    def __init__(self):
        return super().__init__(widgets=[Textarea, AdminFileWidget])

    def render(self, name, value, attrs=None, renderer=None):
        output = []
        try:
            Image.open(value)
        except AttributeError:
            html = ''
        except FileNotFoundError:
            return Textarea(attrs={'cols': 80, 'rows': 1}).render(name, value)
        except UnidentifiedImageError:
            html = u'<a href="{0}{1}" target="_blank">{1}</a>'
        else:
            html = u'<a href="{0}{1}" target="_blank"><img src="{0}{1}" alt="{1}" height="150"  style="object-fit: cover;"/></a>'  # noqa: E501
        output.append(html.format(settings.MEDIA_URL, value))
        output.append(AdminFileWidget().render(name, value, attrs, renderer))
        return mark_safe(u''.join(output))

    def decompress(self, value):
        if isinstance(value, models.fields.files.FieldFile):
            return (None, value)
        else:
            return (value, None)

    def value_from_datadict(self, data, files, name):
        try:
            return files[name]
        except KeyError:
            return data[name]



from django.db import models
from django.contrib import admin
from dynamicfield.models import DynamicField
from dynamicfield.forms import DynamicModelForm
from dynamicfield.admin import DynamicWidget


class MyModel(models.Model):
    INPUT_TYPE_CHOICES = (
        ('Char', 'Text Field (single line)'),
        ('Email', 'Email Field'),
        ('File', 'File Field'),
        ('Image', 'Image Field'),
        ('URL', 'URL Field'),
        ('Text', 'Multiline Text Area'),
    )
    fieldtype = models.CharField(choices=INPUT_TYPE_CHOICES, max_length=10, default='Char')
    value = DynamicField(blank=True, null=True)


class MyModelInline(admin.TabularInline):
    model = MyModel
    form = DynamicModelForm
    formfield_overrides = {
        DynamicField: {'widget': DynamicWidget()},
    }



import ast
from django.db import models
from django.forms.models import model_to_dict


class BorrowerDataBlueprint(models.Model):
    charfield = models.CharField(blank=True, null=True, max_length=50)
    emailfield = models.EmailField(blank=True, null=True)
    filefield = models.FileField(blank=True, null=True, upload_to='uploads/')
    imagefield = models.ImageField(blank=True, null=True, upload_to='uploads/')
    urlfield = models.URLField(blank=True, null=True)
    textfield = models.TextField(blank=True, null=True)

    class Meta:
        abstract = True


class DynamicField(models.TextField):

    def from_db_value(self, value, expression, connection):
        try:
            value = dict(ast.literal_eval(str(value)))
        except (SyntaxError, ValueError, TypeError):
            value = {'charfield': value}
        bb = BorrowerDataBlueprint(**value)
        return (
                bb.charfield
                or bb.emailfield
                or bb.filefield
                or bb.imagefield
                or bb.urlfield
                or bb.textfield
            )

    def pre_save(self, model_instance, add):
        if add or model_instance.value is None:
            return
        sent_value = model_instance.value
        fieldtype = getattr(model_instance, 'fieldtype', 'charfield').lower()
        fieldname = '{}field'.format(fieldtype)
        if model_instance.value == '':
            model_instance.refresh_from_db(fields=['value'])
        bb = BorrowerDataBlueprint(**{fieldname: model_instance.value})
        field = getattr(bb, fieldname)
        if hasattr(field, 'name'):
            if sent_value:
                field.save(field.name, model_instance.value, save=False)
            return {fieldname: field.name}
        else:
            return model_to_dict(bb, fields=[fieldname])

    def to_python(self, value):
        if isinstance(value, dict):
            return super().to_python(value)
        return value
